--[[	*** Historia ***
Written by : Yukyuk, EU-Moonglade
--]]

-------------------------------------------------------------------------------
--  From _Global to local.
-------------------------------------------------------------------------------
local _G = _G


-------------------------------------------------------------------------------
--  AddOn namespace.
-------------------------------------------------------------------------------
local FOLDER_NAME, private = ...

local cfg = private.cfg
local addon = cfg.addon_name


-------------------------------------------------------------------------------
--  Libstubs.
-------------------------------------------------------------------------------
local L 		= LibStub("AceLocale-3.0"):GetLocale("Historia")
local LibQTip 	= LibStub:GetLibrary("LibQTip-1.0")


-------------------------------------------------------------------------------
--	Local constants
-------------------------------------------------------------------------------
local row_count = 0
local Historia_HA_NO_SCROLLBUTTONS = 24			--  number of scroll buttons for the Achievements frame
local Historia_Achievements_Tab = {}			--	this variable holds the selection buttons
local Historia_Achievements_Table = {}			--	this varaible holds the Achievements dates selected from the database
local Historia_Achievements_Scroll_Table = {}	--	this variable holds the selected lines that go into the scroll table


--------------------------------------------------------------------------------
--  This is the Historia achievements frame
--------------------------------------------------------------------------------	
function Historia_Achievements()
	local Historia_HA = CreateFrame("Frame","Historia_HA",Historia_UI)
	Historia_HA:SetID(1)
	Historia_HA:SetPoint("TOP", "Historia_UI", 0, -16)
	Historia_HA:SetPoint("BOTTOM", "Historia_UI", 0, 16)
	Historia_HA:SetPoint("LEFT", "Historia_UI", 16, 0)
	Historia_HA:SetPoint("RIGHT", "Historia_UI", -16, 0)
	Historia_HA:SetBackdrop(cfg.Historia_backdrop3)
	
	--  Create a frame for the header Achievements info.
	local Historia_HA_H = CreateFrame("Frame","Historia_HA_H",Historia_HA)
	Historia_HA_H:SetPoint("TOPLEFT","Historia_HA",0,0)
	Historia_HA_H:SetPoint("BOTTOMRIGHT","Historia_HA","TOPRIGHT",0,-48)
	Historia_HA_H:SetBackdrop(cfg.Historia_backdrop2)	
	
	Historia_HA_H.F1 = Historia_HA_H:CreateFontString(nil,"ARTWORK","GameFontNormalHuge")
	Historia_HA_H.F1:SetPoint("CENTER","Historia_HA_H",0,0)
	Historia_HA_H.F1:SetText(L["History of:"] .. " " .. ACHIEVEMENTS)	
	
	Historia_Achievements_Scrollframe()
	Historia_Achievements_TheRest()	

	Historia_HA:Hide()
end	


-------------------------------------------------------------------------------
--  Fill the Achievements scroll table
-------------------------------------------------------------------------------
function Historia_Achievements_Scrollframe()
	--  Create a reference frame for the Achievements scroll frame.
	local Historia_HA_I = CreateFrame("Frame","Historia_HA_I",Historia_HA)
	Historia_HA_I:SetPoint("TOPLEFT", "Historia_HA_H", "BOTTOMLEFT", 0, -16)
	Historia_HA_I:SetPoint("BOTTOMRIGHT", "Historia_HA", "BOTTOMRIGHT", 0, 0)
	local Historia_HA_I_Width = Historia_HA_I:GetWidth()	
	local Historia_HA_I_Height = Historia_HA_I:GetHeight()

	--  Create a frame for the header Achievements text.
	local Historia_HA_IH = CreateFrame("Frame","Historia_HA_IH",Historia_HA_I)
	Historia_HA_IH:SetPoint("TOPLEFT","Historia_HA_I",0,0)
	Historia_HA_IH:SetPoint("BOTTOMRIGHT","Historia_HA_I","TOPLEFT",540,-24)
	Historia_HA_IH:SetBackdrop(cfg.Historia_backdrop2)	
	
	Historia_HA_IH.F1 = Historia_HA_IH:CreateFontString(nil,"ARTWORK","GameFontWhite")
	Historia_HA_IH.F1:SetPoint("CENTER","Historia_HA_IH",0,0)
	Historia_HA_IH.F1:SetText(ACHIEVEMENTS)	
	
	--  Create the container for the Achievements scroll frame.
	local HASF1 = CreateFrame("Frame","HASF1",Historia_HA_I)
	HASF1:SetPoint("TOPLEFT", "Historia_HA_I", "TOPLEFT", 0, -24)
	HASF1:SetPoint("BOTTOMRIGHT", "Historia_HA_I", "TOPLEFT", 540, Historia_HA_I_Height*-1)
	HASF1:SetBackdrop((cfg.Historia_backdrop2))

	--  Create the container for the Achievements scroll bar.
	local HASF1_bar = CreateFrame("Frame","HASF1_bar",HASF1)
	HASF1_bar:SetPoint("TOPRIGHT", "HASF1", 0, 0)
	HASF1_bar:SetPoint("BOTTOMLEFT", "HASF1", "BOTTOMRIGHT", -24, 0)
	HASF1_bar:SetBackdrop(cfg.Historia_backdrop2)	

	--  Create the list of buttons that go into the Achievements scroll frame
	HASF1.list = {}
	for i=1,Historia_HA_NO_SCROLLBUTTONS do
		HASF1.list[i] = CreateFrame("Button",nil,HASF1)
		HASF1.list[i]:SetSize(540,16)
		HASF1.list[i]:SetPoint("TOPLEFT",HASF1,"TOPLEFT",8,(i-1)*-16-4)
		HASF1.list[i]:Show()
		
		HASF1.list[i].name1 = HASF1.list[i]:CreateFontString(nil,"ARTWORK","GameFontWhite")
		HASF1.list[i].name1:SetSize(160,16)
		HASF1.list[i].name1:SetPoint("LEFT",0,0)	
		HASF1.list[i].name1:SetJustifyH("LEFT") 

		HASF1.list[i].name2 = HASF1.list[i]:CreateFontString(nil,"ARTWORK","GameFontNormal")
		HASF1.list[i].name2:SetSize(380,16)
		HASF1.list[i].name2:SetPoint("LEFT", HASF1.list[i].name1, "RIGHT",4,0)
		HASF1.list[i].name2:SetJustifyH("LEFT") 
	end	
	
	-- 	Create the scroll frame
	HASF1.scrollFrame = CreateFrame("ScrollFrame","HASF1_scrollFrame",HASF1,"FauxScrollFrameTemplate")
	HASF1.scrollFrame:SetPoint("TOPLEFT",HASF1,"TOPLEFT",0,-2)
	HASF1.scrollFrame:SetPoint("BOTTOMRIGHT",HASF1,"BOTTOMRIGHT",-26,2)
	HASF1.scrollFrame:SetScript("OnVerticalScroll",function(self,offset)
	FauxScrollFrame_OnVerticalScroll(self,offset,16,HASF1_ScrollFrameUpdate)
	end)	
end 


-------------------------------------------------------------------------------
--  Fill the Achievements scroll table
--	Fill the extensions part
-------------------------------------------------------------------------------
function Historia_HA_Build_Table(id)
	row_count = 0
	Historia_Achievements_Table = {}
	Historia_Achievements_Scroll_Table = {}	

	for idx = 1, #HistoriaLocalDb.Achievements do
		Historia_Achievements_Table = {}
		row_count = row_count + 1
		
		Historia_Achievements_Table.Text		= HistoriaLocalDb.Achievements[idx].name
		Historia_Achievements_Table.Time		= HistoriaLocalDb.Achievements[idx].time
		tinsert(Historia_Achievements_Scroll_Table,1,Historia_Achievements_Table)
	end	

	for i=1,cfg.Historia_HIA1_Buttons do
		if HistoriaLocalDb.Achievements[cfg.Historia_HIA1_Text[i]] then
			Historia_HA_JI[i].name3:SetText(format(date("%a, %b %d %Y %H:%M",HistoriaLocalDb.Achievements[cfg.Historia_HIA1_Text[i]].time)))
		else
			Historia_HA_JI[i].name3:SetText(L["Not recorded"])
		end
	end	
end	


-------------------------------------------------------------------------------
--  Achievements scroll table the scrolling part.
-------------------------------------------------------------------------------
function HASF1_ScrollFrameUpdate()
	local offset = FauxScrollFrame_GetOffset(HASF1.scrollFrame)
	FauxScrollFrame_Update(HASF1.scrollFrame, row_count, Historia_HA_NO_SCROLLBUTTONS, 16)

	for i = 1, Historia_HA_NO_SCROLLBUTTONS do
		local idx = offset + i
		if idx <= row_count then
			HASF1.list[i].name1:SetText(format(date("%a, %b %d %Y %H:%M", Historia_Achievements_Scroll_Table[idx].Time)))
			HASF1.list[i].name2:SetText(Historia_Achievements_Scroll_Table[idx].Text)
			HASF1.list[i]:Show()
		else
			HASF1.list[i]:Hide()
		end	
	end	
end


-------------------------------------------------------------------------------
--  Handle the Achievements Tab
-------------------------------------------------------------------------------
function Historia_HA_Update()
	Historia_HA_Build_Table()
	
	HASF1.scrollFrame.ScrollBar:SetValue(0)	
	HASF1_ScrollFrameUpdate()	
end


-------------------------------------------------------------------------------
--	Create the rest of the Achievement tab
-------------------------------------------------------------------------------
function Historia_Achievements_TheRest()
	--  Create a reference frame for the rest of the Achievements info.
	local Historia_HA_J = CreateFrame("Frame","Historia_HA_J",Historia_HA)
	Historia_HA_J:SetPoint("TOPLEFT", "Historia_HA_IH","TOPRIGHT",16,0)
	Historia_HA_J:SetPoint("BOTTOMRIGHT", "Historia_HA","BOTTOMRIGHT",0,0)

	local Historia_HA_JH = CreateFrame("Frame","Historia_HA_JH",Historia_HA_J)
	Historia_HA_JH:SetPoint("TOPLEFT","Historia_HA_J",0,0)
	Historia_HA_JH:SetPoint("BOTTOMRIGHT","Historia_HA_J","TOPRIGHT",0,-24)
	Historia_HA_JH:SetBackdrop(cfg.Historia_backdrop2)	
	
	Historia_HA_JH.F1 = Historia_HA_JH:CreateFontString(nil,"ARTWORK","GameFontWhite")
	Historia_HA_JH.F1:SetPoint("CENTER","Historia_HA_JH",0,0)
	Historia_HA_JH.F1:SetText(L["Achievement number"])	

	local Historia_HA_JI = CreateFrame("Frame","Historia_HA_JI",Historia_HA_J)
	Historia_HA_JI:SetPoint("TOPLEFT","Historia_HA_JH","BOTTOMLEFT",0,0)
	Historia_HA_JI:SetPoint("BOTTOMRIGHT","Historia_HA_J",0,0)
	Historia_HA_JI:SetBackdrop(cfg.Historia_backdrop2)

	--  Create the list of buttons
	Historia_HA_JI.list = {}
	for i=1,cfg.Historia_HIA1_Buttons do
		Historia_HA_JI[i] = CreateFrame("Button",nil,Historia_HA_JI)
		Historia_HA_JI[i]:SetSize(200,16)
		Historia_HA_JI[i]:SetPoint("TOPLEFT",Historia_HA_JI,"TOPLEFT",8,(i-1)*-16-4)
		Historia_HA_JI[i]:Show()
		
		Historia_HA_JI[i].name1 = Historia_HA_JI[i]:CreateFontString(nil,"ARTWORK","GameFontWhite")
		Historia_HA_JI[i].name1:SetSize(40,16)
		Historia_HA_JI[i].name1:SetPoint("LEFT",0,0)	
		Historia_HA_JI[i].name1:SetJustifyH("LEFT") 
		Historia_HA_JI[i].name1:SetText(cfg.Historia_HIA1_Text[i])		

		Historia_HA_JI[i].name2 = Historia_HA_JI[i]:CreateFontString(nil,"ARTWORK","GameFontWhite")
		Historia_HA_JI[i].name2:SetSize(20,16)
		Historia_HA_JI[i].name2:SetPoint("LEFT", Historia_HA_JI[i].name1, "RIGHT",4,0)
		Historia_HA_JI[i].name2:SetJustifyH("CENTER")
		Historia_HA_JI[i].name2:SetText(":")	

		Historia_HA_JI[i].name3 = Historia_HA_JI[i]:CreateFontString(nil,"ARTWORK","GameFontNormal")
		Historia_HA_JI[i].name3:SetSize(160,16)
		Historia_HA_JI[i].name3:SetPoint("LEFT", Historia_HA_JI[i].name2, "RIGHT",4,0)
		Historia_HA_JI[i].name3:SetJustifyH("LEFT")				
		
	end		
end

