--[[	*** Historia ***
Written by : Yukyuk, EU-Moonglade
--]]

-------------------------------------------------------------------------------
--  From _Global to local.
-------------------------------------------------------------------------------
local _G = _G

-------------------------------------------------------------------------------
--  AddOn namespace.
-------------------------------------------------------------------------------
local FOLDER_NAME, private = ...

local cfg = private.cfg
local addon = cfg.addon_name


-------------------------------------------------------------------------------
--  Libstubs.
-------------------------------------------------------------------------------
local L 		= LibStub("AceLocale-3.0"):GetLocale("Historia")
local LibQTip 	= LibStub:GetLibrary("LibQTip-1.0")


-------------------------------------------------------------------------------
--	Local variables
-------------------------------------------------------------------------------
local row_count = 0
local Historia_HL_NO_SCROLLBUTTONS = 24		--  number of selection buttons for the levels frame
local Historia_Levels_Table = {}
local Historia_Levels_Scroll_Table = {}	

local Historia_LogosL = {
	[1] = cfg.Icons .."LogoWOW",
	[2] = cfg.Icons .."LogoWOTLK",
	[3] = cfg.Icons .."LogoMP",
	[4] = cfg.Icons .."LogoLEGION",
}

local Historia_LogosR = {
	[1] = cfg.Icons .."LogoBC",
	[2] = cfg.Icons .."LogoCC",
	[3] = cfg.Icons .."LogoWOD",
	[4] = cfg.Icons .."LogoBFA",	
}	
	
local Historia_LevelL = {
	[1] = 60,
	[2] = 80,
	[3] = 90,
	[4] = 110,	
}	
	
local Historia_LevelR = {
	[1] = 70,
	[2] = 85,
	[3] = 100,
	[4] = 120,	
}


--------------------------------------------------------------------------------
--  This is the Historia level frame
--------------------------------------------------------------------------------	
function Historia_Level()
	local Historia_HL = CreateFrame("Frame","Historia_HL",Historia_UI)
	Historia_HL:SetPoint("TOP", "Historia_UI", 0, -16)
	Historia_HL:SetPoint("BOTTOM", "Historia_UI", 0, 16)
	Historia_HL:SetPoint("LEFT", "Historia_UI", 16, 0)
	Historia_HL:SetPoint("RIGHT", "Historia_UI", -16, 0)
	Historia_HL:SetBackdrop(cfg.Historia_backdrop3)
	
	--  Create a frame for the header level info.
	local Historia_HL_H = CreateFrame("Frame","Historia_HL_H",Historia_HL)
	Historia_HL_H:SetPoint("TOPLEFT","Historia_HL",0,0)
	Historia_HL_H:SetPoint("BOTTOMRIGHT","Historia_HL","TOPRIGHT",0,-48)
	Historia_HL_H:SetBackdrop(cfg.Historia_backdrop2)	
	
	Historia_HL_H.F1 = Historia_HL_H:CreateFontString(nil,"ARTWORK","GameFontNormalHuge")
	Historia_HL_H.F1:SetPoint("CENTER","Historia_HL_H",0,0)
	Historia_HL_H.F1:SetText(L["History of:"] .. " " .. L["Levels"])
	
	Historia_Level_Scrollframe()
	Historia_Level_Expansions()

	Historia_HL:Hide()
end	


-------------------------------------------------------------------------------
--  Fill the Level scroll table
-------------------------------------------------------------------------------
function Historia_Level_Scrollframe()
	--  Create a reference frame for the rest of the level info.
	local Historia_HL_I = CreateFrame("Frame","Historia_HL_I",Historia_HL)
	Historia_HL_I:SetPoint("TOPLEFT", "Historia_HL_H", "BOTTOMLEFT", 0, -16)
	Historia_HL_I:SetPoint("BOTTOMRIGHT", "Historia_HL", "BOTTOMRIGHT", 0, 0)
	local Historia_HL_I_Width = Historia_HL_I:GetWidth()	
	local Historia_HL_I_Height = Historia_HL_I:GetHeight()

	--  Create a frame for the header level text.
	local Historia_HL_IH = CreateFrame("Frame","Historia_HL_IH",Historia_HL_I)
	Historia_HL_IH:SetPoint("TOPLEFT","Historia_HL_I",0,0)
	Historia_HL_IH:SetPoint("BOTTOMRIGHT","Historia_HL_I","TOPLEFT",320,-24)
	Historia_HL_IH:SetBackdrop(cfg.Historia_backdrop2)	
	
	Historia_HL_IH.F1 = Historia_HL_IH:CreateFontString(nil,"ARTWORK","GameFontWhite")
	Historia_HL_IH.F1:SetPoint("CENTER","Historia_HL_IH",0,0)
	Historia_HL_IH.F1:SetText(L["Levels"])	
	
	--  Create the container for the levels scroll frame.
	local HLSF1 = CreateFrame("Frame","HLSF1",Historia_HL_I)
	HLSF1:SetPoint("TOPLEFT", "Historia_HL_I", "TOPLEFT", 0, -24)
	HLSF1:SetPoint("BOTTOMRIGHT", "Historia_HL_I", "TOPLEFT", 320, Historia_HL_I_Height*-1)
	HLSF1:SetBackdrop((cfg.Historia_backdrop2))

	--  Create the container for the levels scroll bar.
	local HLSF1_bar = CreateFrame("Frame","HLSF1_bar",HLSF1)
	HLSF1_bar:SetPoint("TOPRIGHT", "HLSF1", 0, 0)
	HLSF1_bar:SetPoint("BOTTOMLEFT", "HLSF1", "BOTTOMRIGHT", -24, 0)
	HLSF1_bar:SetBackdrop(cfg.Historia_backdrop2)	

	--  Create the list of buttons that go into the levels scroll frame
	HLSF1.list = {}
	for i=1,Historia_HL_NO_SCROLLBUTTONS do
		HLSF1.list[i] = CreateFrame("Button",nil,HLSF1)
		HLSF1.list[i]:SetSize(320,16)
		HLSF1.list[i]:SetPoint("TOPLEFT",HLSF1,"TOPLEFT",8,(i-1)*-16-4)
		HLSF1.list[i]:Show()
		
		HLSF1.list[i].name1 = HLSF1.list[i]:CreateFontString(nil,"ARTWORK","GameFontWhite")
		HLSF1.list[i].name1:SetSize(160,16)
		HLSF1.list[i].name1:SetPoint("LEFT",0,0)
		HLSF1.list[i].name1:SetJustifyH("LEFT") 

		HLSF1.list[i].name2 = HLSF1.list[i]:CreateFontString(nil,"ARTWORK","GameFontNormal")
		HLSF1.list[i].name2:SetSize(160,16)
		HLSF1.list[i].name2:SetPoint("LEFT", HLSF1.list[i].name1, "RIGHT",4,0)
		HLSF1.list[i].name2:SetJustifyH("LEFT") 
	end	
	
	-- 	Create the scroll frame
	HLSF1.scrollFrame = CreateFrame("ScrollFrame","HLSF1_scrollFrame",HLSF1,"FauxScrollFrameTemplate")
	HLSF1.scrollFrame:SetPoint("TOPLEFT",HLSF1,"TOPLEFT",0,-2)
	HLSF1.scrollFrame:SetPoint("BOTTOMRIGHT",HLSF1,"BOTTOMRIGHT",-26,2)
	HLSF1.scrollFrame:SetScript("OnVerticalScroll",function(self,offset)
	FauxScrollFrame_OnVerticalScroll(self,offset,16,HLSF1_ScrollFrameUpdate)
	end)	
end 


-------------------------------------------------------------------------------
--  Fill the Level scroll table
--	Fill the extensions part
-------------------------------------------------------------------------------
function Historia_HL_Build_Table()
	row_count = 0
	Historia_Level_Table = {}
	Historia_Level_Scroll_Table = {}	

	for idx = 1, #HistoriaLocalDb.Level do
		Historia_Level_Table = {}
		row_count = row_count + 1
		
		Historia_Level_Table.Level		= HistoriaLocalDb.Level[idx].level
		Historia_Level_Table.Time		= HistoriaLocalDb.Level[idx].time
		tinsert(Historia_Level_Scroll_Table,1,Historia_Level_Table)
		
		if HistoriaLocalDb.Level[idx].level == 60 then
			HLEX1.list1[1].name1:SetText("")
			HLEX1.list1[1].name1:SetText(format(GUILD_NEWS_FORMAT6,cfg.PLAYER_NAME,HistoriaLocalDb.Level[idx].level))		
			HLEX1.list1[1].name2:SetText(format(date("%a, %b %d %Y %H:%M", HistoriaLocalDb.Level[idx].time)))
		elseif HistoriaLocalDb.Level[idx].level == 70 then
			HLEX1.list2[1].name1:SetText("")
			HLEX1.list2[1].name1:SetText(format(GUILD_NEWS_FORMAT6,cfg.PLAYER_NAME,HistoriaLocalDb.Level[idx].level))		
			HLEX1.list2[1].name2:SetText(format(date("%a, %b %d %Y %H:%M", HistoriaLocalDb.Level[idx].time)))		
		elseif HistoriaLocalDb.Level[idx].level == 80 then
			HLEX1.list1[2].name1:SetText("")
			HLEX1.list1[2].name1:SetText(format(GUILD_NEWS_FORMAT6,cfg.PLAYER_NAME,HistoriaLocalDb.Level[idx].level))		
			HLEX1.list1[2].name2:SetText(format(date("%a, %b %d %Y %H:%M", HistoriaLocalDb.Level[idx].time)))
		elseif HistoriaLocalDb.Level[idx].level == 85 then
			HLEX1.list2[2].name1:SetText("")
			HLEX1.list2[2].name1:SetText(format(GUILD_NEWS_FORMAT6,cfg.PLAYER_NAME,HistoriaLocalDb.Level[idx].level))		
			HLEX1.list2[2].name2:SetText(format(date("%a, %b %d %Y %H:%M", HistoriaLocalDb.Level[idx].time)))		
		elseif HistoriaLocalDb.Level[idx].level == 90 then
			HLEX1.list1[3].name1:SetText("")
			HLEX1.list1[3].name1:SetText(format(GUILD_NEWS_FORMAT6,cfg.PLAYER_NAME,HistoriaLocalDb.Level[idx].level))		
			HLEX1.list1[3].name2:SetText(format(date("%a, %b %d %Y %H:%M", HistoriaLocalDb.Level[idx].time)))
		elseif HistoriaLocalDb.Level[idx].level == 100 then
			HLEX1.list2[3].name1:SetText("")
			HLEX1.list2[3].name1:SetText(format(GUILD_NEWS_FORMAT6,cfg.PLAYER_NAME,HistoriaLocalDb.Level[idx].level))		
			HLEX1.list2[3].name2:SetText(format(date("%a, %b %d %Y %H:%M", HistoriaLocalDb.Level[idx].time)))	
		elseif HistoriaLocalDb.Level[idx].level == 110 then
			HLEX1.list1[4].name1:SetText("")
			HLEX1.list1[4].name1:SetText(format(GUILD_NEWS_FORMAT6,cfg.PLAYER_NAME,HistoriaLocalDb.Level[idx].level))		
			HLEX1.list1[4].name2:SetText(format(date("%a, %b %d %Y %H:%M", HistoriaLocalDb.Level[idx].time)))
		elseif HistoriaLocalDb.Level[idx].level == 120 then
			HLEX1.list2[4].name1:SetText("")
			HLEX1.list2[4].name1:SetText(format(GUILD_NEWS_FORMAT6,cfg.PLAYER_NAME,HistoriaLocalDb.Level[idx].level))		
			HLEX1.list2[4].name2:SetText(format(date("%a, %b %d %Y %H:%M", HistoriaLocalDb.Level[idx].time)))				
		end	
	end	
end	


-------------------------------------------------------------------------------
--  Level scroll table the scrolling part.
-------------------------------------------------------------------------------
function HLSF1_ScrollFrameUpdate()
	local offset = FauxScrollFrame_GetOffset(HLSF1.scrollFrame)
	FauxScrollFrame_Update(HLSF1.scrollFrame, row_count, Historia_HL_NO_SCROLLBUTTONS, 16)

	for i = 1, Historia_HL_NO_SCROLLBUTTONS do
		local idx = offset + i
		if idx <= row_count then
			HLSF1.list[i].name1:SetText(format(date("%a, %b %d %Y %H:%M", Historia_Level_Scroll_Table[idx].Time)))
			HLSF1.list[i].name2:SetText(Historia_Level_Scroll_Table[idx].Level)
			HLSF1.list[i]:Show()
		else
			HLSF1.list[i]:Hide()
		end	
	end	
end


-------------------------------------------------------------------------------
--  Handle the Levels Tab
-------------------------------------------------------------------------------
function Historia_HL_Update()
	Historia_HL_Build_Table()
	
	HLSF1.scrollFrame.ScrollBar:SetValue(0)	
	HLSF1_ScrollFrameUpdate()	
end


-------------------------------------------------------------------------------
--	Create and fill the expansions data
-------------------------------------------------------------------------------
function Historia_Level_Expansions()
	--  Create the container for the expansions frame.
	local HLEX1 = CreateFrame("Frame","HLEX1",Historia_HL)
	HLEX1:SetPoint("TOPLEFT", "Historia_HL_IH", "TOPRIGHT", 16, 0)
	HLEX1:SetPoint("BOTTOMRIGHT", "Historia_HL", "BOTTOMRIGHT", 0, 0)
	local Historia_HLEX1_Width = HLEX1:GetWidth()	
	local Historia_HLEX1_Height = HLEX1:GetHeight()
	
	HLEX1.list1 = {}
	--  Create the list1 expansions
	for i=1,4 do
		HLEX1.list1[i] = CreateFrame("Frame",nil,HLEX1)
		HLEX1.list1[i]:SetSize(Historia_HLEX1_Width*0.5,104)
		HLEX1.list1[i]:SetPoint("TOPLEFT",HLEX1,"TOPLEFT",0,(i-1)*-104)
		HLEX1.list1[i]:SetBackdrop((cfg.Historia_backdrop2))		
		HLEX1.list1[i]:Show()
		
		HLEX1.list1[i].icon1 = HLEX1.list1[i]:CreateTexture(nil,"ARTWORK")
		HLEX1.list1[i].icon1:SetSize(128,64)
		HLEX1.list1[i].icon1:SetPoint("TOP",0,-4)
		HLEX1.list1[i].icon1:SetTexture(Historia_LogosL[i])
		
		HLEX1.list1[i].name1 = HLEX1.list1[i]:CreateFontString(nil,"ARTWORK","GameFontNormal")
		HLEX1.list1[i].name1:SetPoint("TOP", HLEX1.list1[i].icon1,"BOTTOM", 0, 0)
		HLEX1.list1[i].name1:SetText(L["Not recorded"])

		HLEX1.list1[i].name2 = HLEX1.list1[i]:CreateFontString(nil,"ARTWORK","GameFontWhite")
		HLEX1.list1[i].name2:SetPoint("TOP", HLEX1.list1[i].name1,"BOTTOM", 0, -4)
	end	

	HLEX1.list2 = {}	
	for i=1,4 do
		HLEX1.list2[i] = CreateFrame("Frame",nil,HLEX1)
		HLEX1.list2[i]:SetSize(Historia_HLEX1_Width*0.5,104)
		HLEX1.list2[i]:SetPoint("TOPRIGHT",HLEX1,"TOPRIGHT",0,(i-1)*-104)
		HLEX1.list2[i]:SetBackdrop((cfg.Historia_backdrop2))		
		HLEX1.list2[i]:Show()
		
		HLEX1.list2[i].icon1 = HLEX1.list2[i]:CreateTexture(nil,"ARTWORK")
		HLEX1.list2[i].icon1:SetSize(128,64)
		HLEX1.list2[i].icon1:SetPoint("TOP",0,-4)
		HLEX1.list2[i].icon1:SetTexture(Historia_LogosR[i])
		
		HLEX1.list2[i].name1 = HLEX1.list2[i]:CreateFontString(nil,"ARTWORK","GameFontNormal")
		HLEX1.list2[i].name1:SetPoint("TOP", HLEX1.list2[i].icon1,"BOTTOM", 0, 0)
		HLEX1.list2[i].name1:SetText(L["Not recorded"])		

		HLEX1.list2[i].name2 = HLEX1.list2[i]:CreateFontString(nil,"ARTWORK","GameFontWhite")
		HLEX1.list2[i].name2:SetPoint("TOP", HLEX1.list2[i].name1,"BOTTOM", 0, -4)
	end		
end
