--[[	*** Historia ***
Written by : Yukyuk, EU-Moonglade
--]]

-------------------------------------------------------------------------------
--  From _Global to local.
-------------------------------------------------------------------------------
local _G = _G
local tinsert, tremove, tsort, twipe = _G.table.insert, _G.table.remove, _G.table.sort, _G.table.wipe


-------------------------------------------------------------------------------
--  AddOn name space.
-------------------------------------------------------------------------------
local FOLDER_NAME, private = ...

local cfg = private.cfg
local addon = cfg.addon_name


-------------------------------------------------------------------------------
--  Libstubs.
-------------------------------------------------------------------------------
local L 		= LibStub("AceLocale-3.0"):GetLocale("Historia")
local LDB		= LibStub("LibDataBroker-1.1")
local LDBIcon 	= LibStub("LibDBIcon-1.0")
local LibQTip 	= LibStub:GetLibrary("LibQTip-1.0")


--  Define the tables for the player professions
local Historia_Professions_init = {}	--	This table holds the initial professions and their levels.
										--	Filled when the players enters the world
local Historia_Professions_current = {}	--  This table holds the current professions and their levels.
										--	We compare this table to the init table so we can detect what has changed.


-------------------------------------------------------------------------------
--  Local functions.
-------------------------------------------------------------------------------
--  Function to split a string into two parts.
-------------------------------------------------------------------------------
local function splitName(splitter,name)
	local part1, part2
	part1, part2 = strsplit(splitter, name, 2)
	return part1, part2
end


-------------------------------------------------------------------------------
--  Check the need to define initial content of dates table.
--  For example: when we pass midnight while playing.
-------------------------------------------------------------------------------
local function HRrecorder_Check_Dates()
	cfg.ADDON_YS = date("%Y")
	cfg.ADDON_MS = date("%m")
	cfg.ADDON_DS = date("%d")

	if not HistoriaLocalDb.Dates or HistoriaLocalDb.Dates[cfg.ADDON_YS] == nil then
		HistoriaLocalDb.Dates[cfg.ADDON_YS] = {}
		HistoriaLocalDb.Dates[cfg.ADDON_YS].Gold_Start = GetMoney()
		HistoriaLocalDb.Dates[cfg.ADDON_YS].Gold_Current = GetMoney()
	end

	if HistoriaLocalDb.Dates[cfg.ADDON_YS][cfg.ADDON_MS] == nil then
		HistoriaLocalDb.Dates[cfg.ADDON_YS][cfg.ADDON_MS] = {}
		HistoriaLocalDb.Dates[cfg.ADDON_YS][cfg.ADDON_MS].Gold_Start = GetMoney()
		HistoriaLocalDb.Dates[cfg.ADDON_YS][cfg.ADDON_MS].Gold_Current = GetMoney()
	end

	if HistoriaLocalDb.Dates[cfg.ADDON_YS][cfg.ADDON_MS][cfg.ADDON_DS] == nil then
		HistoriaLocalDb.Dates[cfg.ADDON_YS][cfg.ADDON_MS][cfg.ADDON_DS] = {}
		HistoriaLocalDb.Dates[cfg.ADDON_YS][cfg.ADDON_MS][cfg.ADDON_DS].Gold_Start = GetMoney()
		HistoriaLocalDb.Dates[cfg.ADDON_YS][cfg.ADDON_MS][cfg.ADDON_DS].Gold_Current = GetMoney()
	end

	if HistoriaLocalDb.Dates[cfg.ADDON_YS][cfg.ADDON_MS][cfg.ADDON_DS].Level == nil then
		HistoriaLocalDb.Dates[cfg.ADDON_YS][cfg.ADDON_MS][cfg.ADDON_DS].Level = {}
	end

	if HistoriaLocalDb.Dates[cfg.ADDON_YS][cfg.ADDON_MS][cfg.ADDON_DS].Professions == nil then
		HistoriaLocalDb.Dates[cfg.ADDON_YS][cfg.ADDON_MS][cfg.ADDON_DS].Professions = {}
	end

	if HistoriaLocalDb.Dates[cfg.ADDON_YS][cfg.ADDON_MS][cfg.ADDON_DS].Achievements == nil then
		HistoriaLocalDb.Dates[cfg.ADDON_YS][cfg.ADDON_MS][cfg.ADDON_DS].Achievements = {}
	end

	if HistoriaLocalDb.Dates[cfg.ADDON_YS][cfg.ADDON_MS][cfg.ADDON_DS].Quests == nil then
		HistoriaLocalDb.Dates[cfg.ADDON_YS][cfg.ADDON_MS][cfg.ADDON_DS].Quests = {}
	end
end


-------------------------------------------------------------------------------
--  Initialize database
-------------------------------------------------------------------------------
function Historia_Database()
	-- This part is the database (Global) for all characters
	if HistoriaGlobalDb == nil then
		HistoriaGlobalDb = {}
	end

	if not HistoriaGlobalDb.UI then
		HistoriaGlobalDb.UI = {
			point = "CENTER",
			relativePoint = "CENTER",
			pos = {
				X = 0,
				Y = 0,
			},
		}
	end

	if not HistoriaGlobalDb[cfg.REALM_NAME] then
		HistoriaGlobalDb[cfg.REALM_NAME] = {}
	end

	if not HistoriaGlobalDb[cfg.REALM_NAME][cfg.PLAYER_NAME] then
		HistoriaGlobalDb[cfg.REALM_NAME][cfg.PLAYER_NAME] = {}
	end

	if not HistoriaGlobalDb[cfg.REALM_NAME][cfg.PLAYER_NAME].CHATMSGSKILL then
		HistoriaGlobalDb[cfg.REALM_NAME][cfg.PLAYER_NAME].CHATMSGSKILL = {}
	end

	if not HistoriaGlobalDb.Database_build then
		HistoriaGlobalDb.Version = 3
		HistoriaGlobalDb.Database_build = time()
		HistoriaGlobalDb.Enter_count = 0
		HistoriaGlobalDb.Leave_time = 0
		HistoriaGlobalDb.Leave_count = 0
		HistoriaGlobalDb.minimapShow = true
	end

	-- This part is the database (Local) for each character
	if HistoriaLocalDb == nil then
		HistoriaLocalDb = {}
	end

	if not HistoriaLocalDb.Database_build then
		HistoriaLocalDb.Version = 8
		HistoriaLocalDb.Enter_count = 0
		HistoriaLocalDb.Database_build = time()
		HistoriaLocalDb.Leave_time = 0
		HistoriaLocalDb.Leave_count = 0
		HistoriaLocalDb.Gold_Current = GetMoney()
		HistoriaLocalDb.Gold_Start = GetMoney()
	end

	--  Update the local database version in case we have an older database.
	if HistoriaLocalDb.Version < 8 then
		HistoriaLocalDb.Version = 8
	end

	--  Create the Dates table
	if not HistoriaLocalDb.Dates then
		HistoriaLocalDb.Dates = {}
		HistoriaLocalDb.isTimeline_currency_checked = true
	end
	--  Check if the date is present
	HRrecorder_Check_Dates()

	if not HistoriaLocalDb.Achievements then
		HistoriaLocalDb.Achievements = {}
		HistoriaLocalDb.isTimeline_achievements_checked = true
	end

	if not HistoriaLocalDb.Level then
		HistoriaLocalDb.Level = {}
		HistoriaLocalDb.isTimeline_level_checked = true
		--  New character, insert character level in the level table
		local plr = {}
		plr.time = time()
		plr.level = UnitLevel("player")
		table.insert(HistoriaLocalDb.Level, plr)
		--  New character, insert new character level in the dates table
		table.insert(HistoriaLocalDb.Dates[cfg.ADDON_YS][cfg.ADDON_MS][cfg.ADDON_DS].Level, plr.level)
	end

	if not HistoriaLocalDb.Professions then
		HistoriaLocalDb.Professions = {}
		HistoriaLocalDb.isTimeline_professions_checked = true
		--  New character, store the professions
		Professions_store_once()
	end

	if not HistoriaLocalDb.Quests then
		HistoriaLocalDb.Quests = {}
		HistoriaLocalDb.isTimeline_quests_checked = true
	end

	if not HistoriaLocalDb.Professions_Conversion1 then
		HistoriaLocalDb.Professions_Conversion1 = false
	end

	if HistoriaLocalDb.Professions_Conversion1 == false then
		for i = 1, #HistoriaLocalDb.Professions do
			HistoriaLocalDb.Professions[i].expansion = NO
		end
		HistoriaLocalDb.Professions_Conversion1 = true
	end
end


-------------------------------------------------------------------------------
--	Handle the Lib Data Broker OnClick event.
-------------------------------------------------------------------------------
function Historia_LDB_OnClick(frame, button)
	if button == "LeftButton" then
		Historia_Toggle_UI()
	end
end


-------------------------------------------------------------------------------
--	Create the Lib Data Broker
-------------------------------------------------------------------------------
function Historia_Create_LDB()
	Historia_LDB = LDB:NewDataObject("Historia", {
			type = "data source",
			text = "Historia",
			icon = [[Interface\Addons\Historia\Icons\HistoriaPurple]],
			OnClick = Historia_LDB_OnClick,
		})

	if not HistoriaLDBIconDB then HistoriaLDBIconDB = {} end
	LDBIcon:Register("HistoriaIcon", Historia_LDB, HistoriaLDBIconDB)
end


-------------------------------------------------------------------------------
--	Initial fill of the professions.
-------------------------------------------------------------------------------
function Professions_store_once()
	-- Capture the SkillIndes for the players Professions
	local prof1, prof2, archaeology, fishing, cooking = GetProfessions()
	-- Build an array of the Professions
	local profs = {prof1, prof2, archaeology, fishing, cooking}
	for k, v in pairs(profs) do
		--  Insert player professions in the profession table
		local name, texture, rank, maxRank, numSpells, spelloffset, skillLine, rankModifier = GetProfessionInfo(v)
		local ppr1 = {}
		ppr1.time = time()
		ppr1.name = name
		ppr1.rank = rank
		ppr1.skillLine = skillLine
		ppr1.expansion = NO
		table.insert(HistoriaLocalDb.Professions, ppr1)

		--  Insert player professions in the dates table
		local ppr2 = {}
		ppr2 = {name, rank, skillLine}
		table.insert(HistoriaLocalDb.Dates[cfg.ADDON_YS][cfg.ADDON_MS][cfg.ADDON_DS].Professions, ppr2)
	end
end


-------------------------------------------------------------------------------
--	Extract the expansion name from the line
-------------------------------------------------------------------------------
local Historia_Expansions = {
	[1] = L["Kul"],
	[2] = L["Zandalari"],
	[3] = L["Legion"],
	[4] = L["Draenor"],
	[5] = L["Pandaria"],
	[6] = L["Cataclysm"],
	[7] = L["Northrend"],
	[8] = L["Outland"],
	[9] = L["Way of the Brew"],	
	[10] = L["Way of the Grill"],
	[11] = L["Way of the Oven"],
	[12] = L["Way of the Pot"],
	[13] = L["Way of the Steamer"],
	[14] = L["Way of the Wok"],
}
local startPos, endPos
local function Historia_Find_expansion(line)
	for i = 1, #Historia_Expansions do
		startPos, endPos = string.find(line,Historia_Expansions[i])
		if startPos ~= nil then
			return(Historia_Expansions[i]) 
		end	
	end	
	return(L["Classic"])
end

local function Historia_Match_rank(line)
	local a = string.match(line,"%d+")
	if a ~= nil then
		return(a) 
	end	
	return(1)
end

local function Historia_Find_name(line)
	for i = 1, #cfg.PTX do
		startPos, endPos = string.find(line,cfg.PTX[i][2])
		if startPos ~= nil then
			return cfg.PTX[i][2],cfg.PTX[i][1] 
		end	
	end	
	return "Error",0
end

-------------------------------------------------------------------------------
--	Store the professions info in the database
-------------------------------------------------------------------------------
function Historia_professions_database()
	--  Copy the current professions record into the database record.
	local DB_name
	local DB_rank
	local DB_skillLine
	local DB_expansion = NO

	--	Find the expansion
	if #HistoriaGlobalDb[cfg.REALM_NAME][cfg.PLAYER_NAME].CHATMSGSKILL == 0 then
		return	
	end
	
	DB_name, DB_skillLine 	= Historia_Find_name(HistoriaGlobalDb[cfg.REALM_NAME][cfg.PLAYER_NAME].CHATMSGSKILL[1])
	if DB_name == "Error" then
		print("Error: ", HistoriaGlobalDb[cfg.REALM_NAME][cfg.PLAYER_NAME].CHATMSGSKILL[1])
		return
	end
	DB_rank					= Historia_Match_rank(HistoriaGlobalDb[cfg.REALM_NAME][cfg.PLAYER_NAME].CHATMSGSKILL[1])
	DB_expansion 			= Historia_Find_expansion(HistoriaGlobalDb[cfg.REALM_NAME][cfg.PLAYER_NAME].CHATMSGSKILL[1])

	--  Insert player professions in the local database professions
	local ppr1 = {}
	ppr1.time = time()
	ppr1.name = DB_name
	ppr1.rank = DB_rank
	ppr1.skillLine = DB_skillLine
	ppr1.expansion = DB_expansion
	table.insert(HistoriaLocalDb.Professions, ppr1)

	--  Insert player professions in the dates table
	local ppr2 = {}
	ppr2 = {DB_name, DB_rank, DB_skillLine}
	table.insert(HistoriaLocalDb.Dates[cfg.ADDON_YS][cfg.ADDON_MS][cfg.ADDON_DS].Professions, ppr2)
	
	HistoriaGlobalDb[cfg.REALM_NAME][cfg.PLAYER_NAME].CHATMSGSKILL = {}
end


-------------------------------------------------------------------------------
--	Handle the CHAT_MSH_SKILL event
-------------------------------------------------------------------------------
local function Historia_Handle_CHATMSGSKILL(self,event,msg)
	table.insert(HistoriaGlobalDb[cfg.REALM_NAME][cfg.PLAYER_NAME].CHATMSGSKILL,1,msg)
end


-------------------------------------------------------------------------------
--	Create and handle the event frame
-------------------------------------------------------------------------------
	local Historia_event_frame = CreateFrame("Frame")
	ChatFrame_AddMessageEventFilter("CHAT_MSG_SKILL",Historia_Handle_CHATMSGSKILL)
	Historia_event_frame:RegisterEvent("ADDON_LOADED");
	Historia_event_frame:RegisterEvent("PLAYER_ENTERING_WORLD")
	Historia_event_frame:RegisterEvent("PLAYER_LEAVING_WORLD")
	Historia_event_frame:RegisterEvent("PLAYER_LEVEL_UP")
	Historia_event_frame:RegisterEvent("PLAYER_MONEY")
	Historia_event_frame:RegisterEvent("ACHIEVEMENT_EARNED")
	Historia_event_frame:RegisterEvent("QUEST_TURNED_IN")

	Historia_event_frame:SetScript("OnEvent", function(self, event, arg1, arg2)
		if event == "ADDON_LOADED" and arg1 == cfg.addon_name then
			Historia_Create_LDB()
			Historia_event_frame:UnregisterEvent("ADDON_LOADED")
		elseif event == "PLAYER_ENTERING_WORLD" then
			--	Remember. We store player data in the database so do NOT move this to the ADDON_LOADED event.
			Historia_Database()
			--  Set the leave count for the global/local database up by 1
			HistoriaGlobalDb.Enter_count = HistoriaGlobalDb.Enter_count + 1
			HistoriaLocalDb.Enter_count = HistoriaLocalDb.Enter_count + 1
			Historia_Create_UI()
			--Historia_professions_table("initial")
			--	This event, SKILL_LINES_CHANGED fires too soon for Historia, thus register here.
			Historia_event_frame:RegisterEvent("SKILL_LINES_CHANGED")
		elseif event == "PLAYER_LEAVING_WORLD" then
			--  Set the leave count for the global/local database up by 1
			HistoriaGlobalDb.Leave_count = HistoriaGlobalDb.Leave_count + 1
			HistoriaLocalDb.Leave_count = HistoriaLocalDb.Leave_count + 1
			--  This is the time stamp of the last player leaving Historia.
			HistoriaGlobalDb.Leave_time = time()
			--  This is the time stamp of the last time a player left Historia.
			HistoriaLocalDb.Leave_time = time()
		elseif event == "PLAYER_LEVEL_UP" then
			HRrecorder_Check_Dates()
			local plr = {}
			plr.time = time()
			plr.level = arg1
			--  Insert new player level in the level table
			table.insert(HistoriaLocalDb.Level, plr)
			--  Insert new player level in the dates table
			table.insert(HistoriaLocalDb.Dates[cfg.ADDON_YS][cfg.ADDON_MS][cfg.ADDON_DS].Level, plr.level)
		elseif event == "PLAYER_MONEY" then
			HRrecorder_Check_Dates()
			HistoriaLocalDb.Gold_Current = GetMoney()
			HistoriaLocalDb.Dates[cfg.ADDON_YS].Gold_Current = GetMoney()
			HistoriaLocalDb.Dates[cfg.ADDON_YS][cfg.ADDON_MS].Gold_Current = GetMoney()
			HistoriaLocalDb.Dates[cfg.ADDON_YS][cfg.ADDON_MS][cfg.ADDON_DS].Gold_Current = GetMoney()
		elseif event == "ACHIEVEMENT_EARNED" then
			HRrecorder_Check_Dates()
			local achievementID = arg1
			local id, name, points, completed, month, day, year, description, flags, icon, rewardText, isGuild, wasEarnedByMe, earnedBy = GetAchievementInfo(achievementID)
			local par1 = {}
			par1.time = time()
			par1.id = id
			par1.name = name
			par1.points = points
			par1.completed = completed
			par1.description = description
			par1.icon = icon
			par1.rewardText = rewardText
			par1.isGuild = isGuild
			par1.wasEarnedByMe = wasEarnedByMe
			par1.earnedBy = earnedBy
			--  Insert new player achievement in the achievement table
			table.insert(HistoriaLocalDb.Achievements, par1)
			--  Insert new player achievement in the dates table
			local par2 = {}
			par2 = {id, name, points, completed, description, icon, rewardText, isGuild, wasEarnedByMe, earnedBy}
			table.insert(HistoriaLocalDb.Dates[cfg.ADDON_YS][cfg.ADDON_MS][cfg.ADDON_DS].Achievements, par2)
		elseif event == "QUEST_TURNED_IN" then
			HRrecorder_Check_Dates()
			local Historia_QuestId = arg1
			local HistoriaQuestText = C_QuestLog.GetQuestInfo(Historia_QuestId)
			local pqr1 = {}
			pqr1.time = time()
			pqr1.Turned_in_questID = Historia_QuestId		-- For the sake of backwards compatibility
			pqr1.Complete_questText = HistoriaQuestText	-- QuestId is stored twice
			pqr1.Complete_questID = Historia_QuestId
			table.insert(HistoriaLocalDb.Quests, pqr1)
			local pqr2= {}
			pqr2 = {Historia_QuestId,  HistoriaQuestText, Historia_QuestId}
			table.insert(HistoriaLocalDb.Dates[cfg.ADDON_YS][cfg.ADDON_MS][cfg.ADDON_DS].Quests, pqr2)
		elseif event == "SKILL_LINES_CHANGED" then
			HRrecorder_Check_Dates()
			Historia_professions_database()
		end
	end)

